@@ -27,6 +27,9 @@ Multimedia settings
 import os
 import gobject
 import gst
+import string
+
+from serializable import Serializable, to_object_from_data_type
 
 from gettext import gettext as _
 
@@ -57,7 +60,8 @@ class GlobalSettings:
         self._readSettings()
 
     def _setDefaults(self):
-        self.advancedModeEnabled = False
+        self.advancedModeEnabled = True
+        self.fileSupportEnabled = False
 
     def _readSettings(self):
         self._readSettingsFromGlobalConfiguration()
@@ -75,9 +79,48 @@ class GlobalSettings:
 
     def _readSettingsFromEnvironmentVariables(self):
         # reads some settings from environment variable
-        self.advancedModeEnabled = get_bool_env("PITIVI_ADVANCED_MODE")
+        #self.advancedModeEnabled =
+        #get_bool_env("PITIVI_ADVANCED_MODE")
+        self.fileSupportEnabled = get_bool_env("PITIVI_FILE_SUPPORT")
+        pass
 
-class ExportSettings(gobject.GObject):
+    def get_local_plugin_path(autocreate=True):
+        """
+        Compute the absolute path to local plugin repository
+
+        @param autocreate: create the path if missing
+        @return: the plugin repository path
+        """
+
+        pitivi_path = os.path.expanduser("~/.pitivi")
+        if not os.path.exists(pitivi_path):
+            os.mkdir(pitivi_path)
+
+        repository_path = os.path.expanduser("~/.pitivi/plugins")
+        if not os.path.exists(repository_path):
+            os.mkdir(repository_path)
+
+        return repository_path
+
+    def get_plugin_settings_path(autocreate=True):
+        """
+        Compute the absolute path to local plugin settings' repository
+
+        @param autocreate: create the path if missing
+        @return: the plugin settings path
+        """
+
+        pitivi_path = os.path.expanduser("~/.pitivi")
+        if not os.path.exists(pitivi_path):
+            os.mkdir(pitivi_path)
+
+        repository_path = os.path.expanduser("~/.pitivi/plugins-settings")
+        if not os.path.exists(repository_path):
+            os.mkdir(repository_path)
+
+        return repository_path
+
+class ExportSettings(gobject.GObject, Serializable):
     """
     Multimedia export settings
 
@@ -95,6 +138,8 @@ class ExportSettings(gobject.GObject):
                                ( ))
         }
 
+    __data_type__ = "export-settings"
+
     # Audio/Video settings for processing/export
 
     # TODO : Add PAR/DAR for video
@@ -201,13 +246,74 @@ class ExportSettings(gobject.GObject):
         if changed:
             self.emit("encoders-changed")
 
+    ## Serializable methods
+
+    def toDataFormat(self):
+        ret = Serializable.toDataFormat(self)
+        ret["video-width"] = self.videowidth
+        ret["video-height"] = self.videoheight
+        ret["video-rate"] = [ self.videorate.num,
+                             self.videorate.denom ]
+        ret["video-par"] = [ self.videopar.num,
+                            self.videopar.denom ]
+        ret["audio-channels"] = self.audiochannels
+        ret["audio-rate"] = self.audiorate
+        ret["audio-depth"] = self.audiodepth
+        ret["video-encoder"] = self.vencoder
+        ret["audio-encoder"] = self.aencoder
+        ret["muxer"] = self.muxer
+        if self.containersettings:
+            ret["container-settings"] = self.containersettings
+        if self.acodecsettings:
+            ret["audio-encoder-settings"] = self.acodecsettings
+        if self.vcodecsettings:
+            ret["video-encoder-settings"] = self.vcodecsettings
+        return ret
+
+    def fromDataFormat(self, obj):
+        Serializable.fromDataFormat(self, obj)
+        self.videowidth = ret["video-width"]
+        self.videoheight = ret["video-height"]
+        self.videorate = gst.Fraction(*ret["video-rate"])
+        self.videopar = gst.Fraction(*ret["video-par"])
+
+        self.audiochannels = ret["audio-channels"]
+        self.audiorate = ret["audio-rate"]
+        self.audiodepth = ret["audio-depth"]
+
+        # FIXME : check if the given encoder/muxer are available
+        self.vencoder = ret["video-encoder"]
+        self.aencoder = ret["audio-encoder"]
+        self.muxer = ret["muxer"]
+        if "container-settings" in ret:
+            self.containersettings = ret["container-settings"]
+        if "audio-encoder-settings" in ret:
+            self.acodecsettings = ret["audio-encoder-settings"]
+        if "video-encoder-settings" in ret:
+            self.vcodecsettings = ret["video-encoder-settings"]
+
+def list_compat(a, b):
+    for x in a:
+        if not x in b:
+            return False
+    return True
+
+def my_can_sink_caps(muxer, ocaps):
+    """ returns True if the given caps intersect with some of the muxer's
+    sink pad templates' caps.
+    """
+    sinkcaps = [x.get_caps() for x in muxer.get_static_pad_templates() if x.direction == gst.PAD_SINK]
+    for x in sinkcaps:
+        if not x.intersect(ocaps).is_empty():
+            return True
+    return False
 
 def available_muxers():
     """ return all available muxers """
     flist = gst.registry_get_default().get_feature_list(gst.ElementFactory)
     res = []
     for fact in flist:
-        if "Codec/Muxer" == fact.get_klass():
+        if list_compat(["Codec", "Muxer"], string.split(fact.get_klass(), '/')):
             res.append(fact)
     gst.log(str(res))
     return res
@@ -217,7 +323,9 @@ def available_video_encoders():
     flist = gst.registry_get_default().get_feature_list(gst.ElementFactory)
     res = []
     for fact in flist:
-        if "Codec/Encoder/Video" in fact.get_klass():
+        if list_compat(["Codec", "Encoder", "Video"], string.split(fact.get_klass(), '/')):
+            res.append(fact)
+        elif list_compat(["Codec", "Encoder", "Image"], string.split(fact.get_klass(), '/')):
             res.append(fact)
     gst.log(str(res))
     return res
@@ -227,7 +335,7 @@ def available_audio_encoders():
     flist = gst.registry_get_default().get_feature_list(gst.ElementFactory)
     res = []
     for fact in flist:
-        if "Codec/Encoder/Audio" in fact.get_klass():
+        if list_compat(["Codec", "Encoder", "Audio"], string.split(fact.get_klass(), '/')):
             res.append(fact)
     gst.log(str(res))
     return res
@@ -238,7 +346,7 @@ def encoders_muxer_compatible(encoders, muxer):
     res = []
     for encoder in encoders:
         for caps in [x.get_caps() for x in encoder.get_static_pad_templates() if x.direction == gst.PAD_SRC]:
-            if muxer.can_sink_caps(caps):
+            if my_can_sink_caps(muxer, caps):
                 res.append(encoder)
                 break
     return res
