@@ -46,6 +46,7 @@ from sourcefactories import SourceFactoriesWidget
 from viewer import PitiviViewer
 from pitivi.bin import SmartTimelineBin
 from projectsettings import ProjectSettingsDialog
+from pluginmanagerdialog import PluginManagerDialog
 from pitivi.configure import pitivi_version, APPNAME
 from glade import GladeWindow
 
@@ -76,9 +77,11 @@ class PitiviMainWindow(gtk.Window):
         self.isFullScreen = False
         self.errorDialogBox = None
 
-        instance.PiTiVi.connect("new-project", self._newProjectCb)
+        instance.PiTiVi.connect("new-project-loaded", self._newProjectCb)
         instance.PiTiVi.connect("closing-project", self._closingProjectCb)
-        instance.PiTiVi.connect("not-project", self._notProjectCb)
+        instance.PiTiVi.connect("new-project-failed", self._notProjectCb)
+        instance.PiTiVi.current.connect("save-uri-requested", self._saveAsDialogCb)
+        instance.PiTiVi.current.connect("confirm-overwrite", self._confirmOverwriteCb)
         instance.PiTiVi.playground.connect("error", self._playGroundErrorCb)
         instance.PiTiVi.current.sources.connect_after("file_added", self._sourcesFileAddedCb)
 
@@ -98,6 +101,8 @@ class PitiviMainWindow(gtk.Window):
 
     def _timelineDurationChangedCb(self, unused_composition, unused_start,
                                    duration):
+        if not isinstance(instance.PiTiVi.playground.current, SmartTimelineBin):
+            return
         self.render_button.set_sensitive((duration > 0) and True or False)
         if duration > 0 :
             gobject.idle_add(self.timeline.simpleview._displayTimeline)
@@ -107,7 +112,8 @@ class PitiviMainWindow(gtk.Window):
 	    self.render_button.set_sensitive(False)
         else:
             if isinstance(smartbin, SmartTimelineBin):
-                gst.info("switching to Timeline, setting duration to %s" % (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
+                gst.info("switching to Timeline, setting duration to %s" %
+                         (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
                 smartbin.project.timeline.videocomp.connect("start-duration-changed",
                                                             self._timelineDurationChangedCb)
                 if smartbin.project.timeline.videocomp.duration > 0:
@@ -144,10 +150,12 @@ class PitiviMainWindow(gtk.Window):
             ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project settings"), None, _("Edit the project settings"), self._projectSettingsCb),
             ("ImportSources", gtk.STOCK_ADD, _("_Import clips..."), None, _("Import clips to use"), self._importSourcesCb),
             ("ImportSourcesFolder", gtk.STOCK_ADD, _("_Import folder of clips..."), None, _("Import folder of clips to use"), self._importSourcesFolderCb),
-	    ("RenderProject", 'pitivi-render' , _("_Render project"), None, _("Render project"), self._recordCb),
+            ("RenderProject", 'pitivi-render' , _("_Render project"), None, _("Render project"), self._recordCb),
+            ("PluginManager", gtk.STOCK_PREFERENCES , _("_Plugins..."), None, _("Manage plugins"), self._pluginManagerCb),
             ("Quit", gtk.STOCK_QUIT, None, None, None, self._quitCb),
             ("About", gtk.STOCK_ABOUT, None, None, _("Information about %s") % APPNAME, self._aboutCb),
             ("File", None, _("_File")),
+            ("Edit", None, _("_Edit")),
             ("View", None, _("_View")),
             ("Help", None, _("_Help"))
             ]
@@ -166,13 +174,17 @@ class PitiviMainWindow(gtk.Window):
         for action in self.actiongroup.list_actions():
             if action.get_name() == "RenderProject":
                 self.render_button = action
-            if action.get_name() == "AdvancedView":
+            elif action.get_name() == "AdvancedView":
                 if not instance.PiTiVi.settings.advancedModeEnabled:
                     action.set_visible(False)
-            if action.get_name() in ["ProjectSettings", "Quit", "File", "Help",
+            elif action.get_name() in ["ProjectSettings", "Quit", "File", "Edit", "Help",
                                      "About", "View", "FullScreen", "ImportSources",
-                                     "ImportSourcesFolder", "AdvancedView"]:
+                                     "ImportSourcesFolder", "AdvancedView", "PluginManager"]:
                 action.set_sensitive(True)
+            elif action.get_name() in ["SaveProject", "SaveProjectAs",
+                                       "NewProject"]:
+                if not instance.PiTiVi.settings.fileSupportEnabled:
+                    action.set_sensitive(False)
             else:
                 action.set_sensitive(False)
 
@@ -277,16 +289,16 @@ class PitiviMainWindow(gtk.Window):
     ## Toolbar/Menu actions callback
 
     def _newProjectMenuCb(self, unused_action):
-        instance.PiTiVi.new_blank_project()
+        instance.PiTiVi.newBlankProject()
 
     def _openProjectCb(self, unused_action):
         raise NotImplementedError
 
     def _saveProjectCb(self, unused_action):
-        raise NotImplementedError
+        instance.PiTiVi.current.save()
 
     def _saveProjectAsCb(self, unused_action):
-        raise NotImplementedError
+        instance.PiTiVi.current.saveAs()
 
     def _projectSettingsCb(self, unused_action):
         l = ProjectSettingsDialog(self, instance.PiTiVi.current)
@@ -312,9 +324,17 @@ class PitiviMainWindow(gtk.Window):
         abt.set_name(APPNAME)
         abt.set_version("v%s" % pitivi_version)
         abt.set_website("http://www.pitivi.org/")
-        authors = ["Edward Hervey <edward@fluendo.com>","",_("Contributors:"),
+        authors = ["Edward Hervey <bilboed@bilboed.com>","",_("Contributors:"),
                    "Christophe Sauthier <christophe.sauthier@gmail.com> (i18n)",
-                   "Laszlo Pandy <laszlok2@gmail.com> (UI)", ]
+                   "Laszlo Pandy <laszlok2@gmail.com> (UI)",
+                   "Ernst Persson  <ernstp@gmail.com>",
+                   "Richard Boulton <richard@tartarus.org>",
+                   "Thibaut Girka <thibaut.girka@free.fr> (UI)",
+                   "Jeff Fortin <nekohayo@gmail.com> (UI)",
+                   "Johan Dahlin <jdahlin@async.com.br> (UI)",
+                   "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)",
+                   "Luca Della Santina <dellasantina@farm.unipi.it>",
+                   "Thijs Vermeir <thijsvermeir@gmail.com>"]
         abt.set_authors(authors)
         abt.set_license(_("GNU Lesser General Public License\nSee http://www.gnu.org/copyleft/lesser.html for more details"))
         abt.set_icon_from_file(configure.get_global_pixmap_dir() + "/pitivi.png")
@@ -327,21 +347,80 @@ class PitiviMainWindow(gtk.Window):
     def _importSourcesFolderCb(self, unused_action):
         self.sourcefactories.sourcelist.showImportSourcesDialog(True)
 
+    def _pluginManagerCb(self, unused_action):
+        PluginManagerDialog(instance.PiTiVi.plugin_manager)
+
     ## PiTiVi main object callbacks
 
     def _newProjectCb(self, pitivi, project):
-        raise NotImplementedError
+        pass
 
     def _closingProjectCb(self, pitivi, project):
-        # Return True if we accept the project being close
-        # if we want to save it before it being closed, we must
-        #   do so
+        if not project.hasUnsavedModifications():
+            return True
+        
+        dialog = gtk.MessageDialog(
+            self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_QUESTION,
+            gtk.BUTTONS_YES_NO,
+            _("The project has unsaved changes. Do you wish to close the project?"))
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_YES:
+            return True
+        return False
 
-        # For the time being we always accept it being closed
-        return True
+    def _notProjectCb(self, pitivi, reason, uri):
+        dialog = gtk.MessageDialog(self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_ERROR,
+            gtk.BUTTONS_OK,
+            _("PiTiVi is unable to load file \"%s\": %s") %
+                (uri, reason))
+        dialog.set_title(_("Error Loading File"))
+        dialog.run()
+        dialog.destroy()           
 
-    def _notProjectCb(self, pitivi, uri):
-        raise NotImplementedError
+    ## PiTiVi current project callbacks
+
+    def _confirmOverwriteCb(self, unused_project, uri):
+        message = _("Do you wish to overwrite existing file \"%s\"?") %\
+                 gst.uri_get_location(uri)
+        
+        dialog = gtk.MessageDialog(self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_WARNING,
+            gtk.BUTTONS_YES_NO,
+            message)
+        
+        dialog.set_title(_("Overwrite Existing File?"))
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_YES:
+            return True
+        return False
+
+    def _saveAsDialogCb(self, unused_project):
+        chooser = gtk.FileChooserDialog(_("Save As..."),
+            self,
+            action=gtk.FILE_CHOOSER_ACTION_SAVE,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+            gtk.STOCK_SAVE, gtk.RESPONSE_OK))
+        chooser.set_select_multiple(False)
+        chooser.set_current_name(_("Untitled"))
+        chooser.set_default_response(gtk.RESPONSE_CANCEL)
+        #TODO: get appropriate filters from projectsaver module
+
+        response = chooser.run()
+  
+        if response == gtk.RESPONSE_OK:
+            fn = "file://" + chooser.get_filename()
+        else:
+            fn = None
+        chooser.destroy()
+        return fn
+        
 
 class EncodingDialog(GladeWindow):
     """ Encoding dialog box """
@@ -392,9 +471,10 @@ class EncodingDialog(GladeWindow):
 
     def _positionCb(self, unused_playground, unused_smartbin, position):
         timediff = time.time() - self.timestarted
-        self.progressbar.set_fraction(float(position) / float(self.bin.length))
-        if timediff > 5.0:
-            # only display ETA after 5s in order to have enough averaging
+        self.progressbar.set_fraction(float(min(position, self.bin.length)) / float(self.bin.length))
+        if timediff > 5.0 and position:
+            # only display ETA after 5s in order to have enough averaging and
+            # if the position is non-null
             totaltime = (timediff * float(self.bin.length) / float(position)) - timediff
             self.progressbar.set_text(_("Finished in %dm%ds") % (int(totaltime) / 60,
                                                               int(totaltime) % 60))
